# -*- coding: utf-8 -*-

# Standard Library
import datetime
from dateutil.relativedelta import relativedelta

# Vendor Library
from sqlalchemy import and_, or_
from sqlalchemy.orm import load_only

# Project Library
from kinder.core.declaration_status.models import DSS
from kinder.core.dict.models import WorkType
from kinder.core.dict.models import GroupTypeEnumerate
from gisdo.alchemy_models import (
    AlchemyGroupType, AlchemyGroupWorkType,
    AlchemyDeclarationStatus, AlchemyDeclaration, AlchemyChildren)
from gisdo.algorithm import BaseEnrolledAlgorithm
from gisdo.constants import ALL
from gisdo.utils import AgeDeltas


class EnrolledCountByAge(BaseEnrolledAlgorithm):
    """Кол-во зачислений в Ф группы, без временно отчисленных детей"""

    def get_result_set(self, **kwargs):

        not_temporary_deduct_pupils = super(
            EnrolledCountByAge, self).get_result_set()
        fact_groups = self._get_fact_group()
        kids = self._kids(**kwargs)

        return (
            self.session.query(
                not_temporary_deduct_pupils)
            .join(
                fact_groups,
                fact_groups.c.id == not_temporary_deduct_pupils.c.grup_id)
            .join(
                kids,
                kids.c.id == not_temporary_deduct_pupils.c.children_id)
            .subquery()
        )


class PredictableDecreaseContingent(EnrolledCountByAge):
    """Прогнозируемое уменьшение контингента

    - количество детей в текущих группах (реестр "Группы")
      возраст которых больше 6,5 лет
    - дата расчета возраста = текущая дата
    """

    def _kids(self, **kwargs):

        today = datetime.date.today()
        # Если ДР раньше этой даты, то ребенку > 6,5 лет.
        date_of_birth_border = today - relativedelta(years=6, months=6)

        kids = (
            self.session.query(
                AlchemyChildren).filter(
                    AlchemyChildren.date_of_birth < date_of_birth_border)
            .options(load_only('id'))
            .subquery())

        return kids


class EnrolledCountByCare(BaseEnrolledAlgorithm):
    """Группы по присмотру и уходу"""

    def get_result_set(self, **kwargs):

        not_temporary_deduct_pupils = super(
            EnrolledCountByCare, self).get_result_set()
        fact_groups = self._get_fact_group()
        kids = self._kids(**kwargs)

        return (
            self.session.query(
                not_temporary_deduct_pupils)
            .join(
                fact_groups,
                and_(
                    fact_groups.c.id ==
                    not_temporary_deduct_pupils.c.grup_id,
                    fact_groups.c.is_care))
            .join(
                AlchemyGroupType,
                and_(
                    fact_groups.c.type_id ==
                    AlchemyGroupType.id,
                    AlchemyGroupType.code == GroupTypeEnumerate.DEV))
            .join(
                kids,
                kids.c.id == not_temporary_deduct_pupils.c.children_id)
            .subquery()
        )


class EnrolledCountByDisabled(BaseEnrolledAlgorithm):

    def get_result_set(self, **kwargs):

        fact_group_this_unit = self._get_fact_group()
        not_temporary_deduct_pupils = super(
            EnrolledCountByDisabled, self).get_result_set()
        kids = self._kids(**kwargs)

        return (
            self.session.query(
                not_temporary_deduct_pupils)
            .join(
                fact_group_this_unit,
                fact_group_this_unit.c.id ==
                not_temporary_deduct_pupils.c.grup_id)
            .join(
                AlchemyGroupType,
                and_(
                    fact_group_this_unit.c.type_id ==
                    AlchemyGroupType.id, or_(
                        AlchemyGroupType.code == GroupTypeEnumerate.COMP,
                        AlchemyGroupType.code == GroupTypeEnumerate.COMBI)))
            .join(
                kids,
                kids.c.id == not_temporary_deduct_pupils.c.children_id)
            .subquery())


class EnrolledCountByFamily(BaseEnrolledAlgorithm):

    def get_result_set(self, **kwargs):

        fact_group_this_unit = self._get_fact_group()
        not_temporary_deduct_pupils = super(
            EnrolledCountByFamily, self).get_result_set()
        kids = self._kids(**kwargs)

        return (
            self.session.query(
                not_temporary_deduct_pupils)
            .join(
                fact_group_this_unit,
                fact_group_this_unit.c.id ==
                not_temporary_deduct_pupils.c.grup_id)
            .join(
                kids,
                kids.c.id == not_temporary_deduct_pupils.c.children_id)
            .join(
                AlchemyGroupType,
                and_(
                    fact_group_this_unit.c.type_id ==
                    AlchemyGroupType.id,
                    AlchemyGroupType.code == GroupTypeEnumerate.DEV))
            .filter(fact_group_this_unit.c.is_family)
            .subquery())


class EnrolledCountByCorrect(BaseEnrolledAlgorithm):

    def get_result_set(self, **kwargs):

        fact_group_this_unit = self._get_fact_group()
        not_temporary_deduct_pupils = super(
            EnrolledCountByCorrect, self).get_result_set()
        kids = self._kids(**kwargs)

        return (
            self.session.query(
                not_temporary_deduct_pupils)
            .join(
                fact_group_this_unit,
                fact_group_this_unit.c.id ==
                not_temporary_deduct_pupils.c.grup_id)
            .join(
                AlchemyGroupType,
                and_(
                    fact_group_this_unit.c.type_id == AlchemyGroupType.id,
                    AlchemyGroupType.code == 'compensative'))
            .join(
                kids,
                kids.c.id == not_temporary_deduct_pupils.c.children_id)
            .subquery())


class EnrolledCountByHealth(BaseEnrolledAlgorithm):

    def get_result_set(self, **kwargs):

        fact_group_this_unit = self._get_fact_group()
        not_temporary_deduct_pupils = super(
            EnrolledCountByHealth, self).get_result_set()
        kids = self._kids(**kwargs)

        return (
            self.session.query(
                not_temporary_deduct_pupils)
            .join(
                fact_group_this_unit,
                fact_group_this_unit.c.id ==
                not_temporary_deduct_pupils.c.grup_id)
            .join(
                AlchemyGroupType,
                and_(
                    fact_group_this_unit.c.type_id == AlchemyGroupType.id,
                    AlchemyGroupType.code == GroupTypeEnumerate.HEALTH))
            .join(
                kids,
                kids.c.id == not_temporary_deduct_pupils.c.children_id)
            .subquery()
        )


class EnrolledCountStay(BaseEnrolledAlgorithm):
    def get_result_set(self, group_work_type=None, **kwargs):
        fact_group_this_unit = self._get_fact_group()
        not_temporary_deduct_pupils = super(
            EnrolledCountStay, self).get_result_set()
        kids = self._kids(**kwargs)

        return (
            self.session.query(
                not_temporary_deduct_pupils)
            .join(
                fact_group_this_unit,
                fact_group_this_unit.c.id ==
                not_temporary_deduct_pupils.c.grup_id)
            .join(
                AlchemyGroupWorkType,
                and_(
                    fact_group_this_unit.c.work_type_id ==
                    AlchemyGroupWorkType.id,
                    AlchemyGroupWorkType.code == group_work_type))
            .join(
                kids,
                kids.c.id == not_temporary_deduct_pupils.c.children_id)
            .subquery())


class EnrolledCountShortStay(EnrolledCountStay):
    def get_result_set(self, **kwargs):
        return super(EnrolledCountShortStay, self).get_result_set(
            WorkType.SHORT, **kwargs)


class EnrolledCountFullDayStay(EnrolledCountStay):
    def get_result_set(self, **kwargs):
        return super(EnrolledCountFullDayStay, self).get_result_set(
            WorkType.ALLDAY, **kwargs)


class EnrolledCountWantChangeDou(BaseEnrolledAlgorithm):

    def get_result_set(self, **kwargs):

        fact_group_this_unit = self._get_fact_group()
        not_temporary_deduct_pupils = super(
            EnrolledCountWantChangeDou, self).get_result_set()
        kids = self._kids(**kwargs)
        pupils = self.session.query(
            not_temporary_deduct_pupils).join(
                fact_group_this_unit).subquery()
        pupils = (
            self.session.query(
                kids.c.id.label(
                    'children_id'))
            .join(
                pupils, pupils.c.children_id == kids.c.id)
            .subquery())

        want_change_dou_status = self.session.query(
            AlchemyDeclarationStatus).filter(
                AlchemyDeclarationStatus.code ==
                DSS.WANT_CHANGE_DOU).subquery()
        declarations_want_change_dou = (
            self.session.query(
                AlchemyDeclaration)
            .join(want_change_dou_status)
            .join(pupils,
                  AlchemyDeclaration.children_id ==
                  pupils.c.children_id)
            .options(load_only('children_id'))
            .distinct()
            .subquery())

        return declarations_want_change_dou


class EnrolledCountWantChangeDouWithHN(EnrolledCountWantChangeDou):

    def _kids(self, **kwargs):

        age_range = kwargs.get('age_range', ALL)
        age_on_date = kwargs.get('on_date', AgeDeltas.calculate_date)
        down, up = AgeDeltas.get_category_deltas(age_range, age_on_date)
        kids = (
            self.session.query(
                AlchemyChildren).filter(
                    AlchemyChildren.date_of_birth > down,
                    AlchemyChildren.date_of_birth <= up,
                    AlchemyChildren.health_need_id != None)
            .options(load_only('id'))
            .subquery())
        return kids
